	   .title "Adaptive Filter"
	   .def   ADPFIR
	   .def   X,Y
	   .mmregs

*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
* This 128-tap adaptive FIR filter uses on-chip memory block B0 for
* coefficients and block B1 for data samples.  The newest input should
* be in memory location X when called.	The output will be in memory location Y
* when returned.
* OVLY =1 , RAM =1 when this routine is called.
*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COEFFP .set    02000h	   ; Program Memory Address of the coeff. in S/A RAM
COEFFD .set    02000h	   ; Data memory address of the coeff. in S/A RAM
;;;;	For TMS320C51, COEFFD is 0800h instead of 02000h

ONE    .set    7Ah	   ; Constant One.	       (DP=0).
BETA   .set    7Bh	   ; Adaptation constant.      (DP=0).
ERR    .set    7Ch	   ; Signal error.	       (DP=0).
ERRF   .set    7Dh	   ; Error function.	       (DP=0).
Y      .set    7Eh	   ; Filter output.	       (DP=0).
X      .set    037Fh	   ; Newest Data sample.
FRSTAP .set    0380h	   ; Next newest data sample.
LASTAP .set    03FFh	   ; Oldest data sample.



*
* Finite impulse response (FIR) filter.
*

ADPFIR	   ZPR		   ; Clear P register.
           LACC    ONE,14  ; Load output rounding bit.
	   MAR	   *,AR3
	   LAR	   AR3,#LASTAP ; Point to oldest sample.
FIR	   RPT	   #127
	   MACD    COEFFP,*-   ; 128-tap FIR filter.
	   APAC
	   SACH    Y,1	   ; Store the filter output.
	   NEG		   ; Acc = -y(n)
	   LAR	   AR3,#X
	   ADD	   *,15    ; Add the newest input sample.
	   SACH    ERR,1   ; err(n) = x(n) - y(n)
	   DMOV    *	   ; Include newest sample
*
* LMS Adaption of Filter Coefficients.
*
	   LT	   ERR	   ; T = err
	   MPY	   BETA    ; P = beta*err(i)
	   PAC		   ; errf(i) = beta * err(i)
	   ADD	   ONE,14  ; Round the results.
	   SACH    ERRF,1  ; Save errf(i)
*
	   LACC    #126
	   SAMM    BRCR        ; 127 coefficients to update
*				 in the loop.
	   LAR	   AR2,#COEFFD ; Point to the coefficients.
	   LAR	   AR3,#LASTAP ; Point to the data samples.
	   LT	   ERRF
	   MPY	   *-,AR2      ; P = 2*beta*err(i)*x(i-255)
*
	   RPTB    LOOP-1      ; For I=0,I<=126,I++
ADAPT	   ZALR    *,AR3       ; Load ACCH with ak(i).
	   MPYA    *-,AR2      ; P = 2*beta*err(i)*x(i-k-1)
*				 Acc = ak(i) + 2*beta*err(i)*x(i-k)
	   SACH    *+	       ; Store ak(i+1)
*
LOOP	   ZALR    *,AR3       ; Finally update last coeff. a0(i)

	   RETD 	       ; Delayed return
	   APAC 	       ; Acc = a0(i) + 2*beta*err(i)*x(i)
	   SACH    *+	       ; Save a0(i+1)
