;***************************************************************************
;*                     GO DSP CORPORATION                                  *
;*                     (c) Copyright 1994                                  *
;***************************************************************************
;***************************************************************************
;*                    Amplitude Modulation Demo                            *
;***************************************************************************


;******************* Reset & interrupt vectors ******************************

                .ps 0a00h
                .ds 0e00h
pLPFCoef:

        .data
LPFCoef:        ; Low Pass Filter Coefficients
 .word -1608
 .word -2631
 .word -837
 .word -213
 .word 2197
 .word 2962
 .word 2534
 .word -264
 .word -3770
 .word -6290
 .word -5313
 .word 223
 .word 9759
 .word 20619
 .word 29250
 .word 32521
 .word 29250
 .word 20619
 .word 9759
 .word 223
 .word -5313
 .word -6290
 .word -3770
 .word -264
 .word 2534
 .word 2962
 .word 2197
 .word -213
 .word -837
 .word -2631
 .word -1608

; Data Labels are used to generate symbolic information for the VDE

        .ds  0e80h
sinBuf  ; buffer for sine wave
        .word   0
        .space  0FF0h
sigBuf  ; buffer for processing input signal
        .word   0
        .space  0FF0h
       .ds  0b80h
alpha   ; Modulation Factor
        .word 50h
beta    ; exponential decay for envelope detector
        .word 7800h
sineFreq        ; cos(f) cosine of the modulation frequency
        .word 0B564h
sineAmp     ; Gain for sine wave
        .word 4000h
bufSize  ; size of signal buffer before interpolation
        .word 64
interFct ; Interpolation Factor minus 2
        .word 2
intBufSz ; size of signal buffer after interpolation (calculated)
        .word 0
sDlyLine
        .word 0
        .word 7FFFh
lastPeak     ; Delay line for PeaknLeak subroutine
        .word 0
dc     ; DC component of output signal
        .word 0
temp ; temporary variable
        .word 0

intDlyLn ; delay line for interpolation filter
        .word 0
        .space  1F0h
lpfDlyLn   ; delay line for Low Pass filter
        .word 0
        .space  1F0h
        .text
        .entry                          ; Set entry point for program
Init    ; Initialization
        LDP     #23
        SPM      0

		LAMM 7
		AND	#0FFFBh		; for compatibilty mode set to NDX=0
		SAMM 7
        SETC OVM
        SETC SXM
        MAR     *, AR1
        LACL #0
        LAR AR1, #sinBuf
        RPT     #255
        SACL *+
        LAR AR1, #sigBuf
        RPT  #255
        SACL *+
        LAR AR1, #intDlyLn
        RPT     #31
        SACL *+
        LAR AR1, #lpfDlyLn
        RPT     #31
        SACL *+
        CALL CalBufLn
        LAR AR1,#sDlyLine
        CALL InitSGen,*,AR1
Main
        NOP
Stage1 ; Generate Sine Wave for Modulation
        LAR  AR0,intBufSz
        LAR AR1,#sinBuf
        LAR AR2,#sDlyLine
        MAR *, AR0
        CALL SineGen,*-,AR2
Stage2 ; Interpolate Input
        LAR  AR0,bufSize
        LAR AR1,#sigBuf
        MAR     *,AR0
        CALL Intplate,*-,AR1
Stage3 ; Amplitude Modulate Input
        LAR  AR0,intBufSz
        LAR AR1,#sigBuf
        LAR AR2,#sinBuf
        MAR *,AR0
        CALL Modlator,*-,AR2

Stage4 ; Rectify Signal
        LAR  AR0, intBufSz
        LAR  AR1,#sigBuf
        MAR  *,AR0
        CALL Rectify,*-,AR1

Stage5 ; Envelope Detector
        LAR  AR0,intBufSz
        LAR  AR1,#sigBuf
        MAR *,AR0
        CALL PeakLeak,*-,AR1

Stage6 ; Low Pass Filter Signal
        LAR  AR0,intBufSz
        LAR AR1,#sigBuf
        LAR AR2,#lpfDlyLn
        MAR *,AR0
        CALL FIR,*-,AR1

Stage7 ; Remove DC component
        LAR  AR0,intBufSz
        LAR  AR1,#sigBuf
        MAR  *, AR0
        CALL RemoveDC,*-,AR1

;Stage7 ; Decimate Signal
;       LAR  AR0,intBufSz
;       LAR  AR1,#sigBuf
;       MAR   *,AR0
;       CALL Decimate,*-,AR1
        B       Main

;*************************************************************************
;*  Initialize Delay Line for Sine Wave Generator
;*  ---------------------------------------------
;*
;*  Input: ARP = AR1
;*         AR1 = IIR delay line
;*************************************************************************
InitSGen:    ; Initialize Delay Line for Sine Wave Generator
        LACC  sineFreq,15
        SACH *+
        LACC #04000h
        SACL *-
        RET

;*************************************************************************
;*  Sine Wave Generator Subroutine
;*  ------------------------------
;*  Input: AR0 = number of samples - 1
;*         AR1 = start of output buffer
;*             AR2 = IIR stage delay line
;*         ARP = AR2
;*
;*  y[n] = 2cos(f)*y[n-1] - y[n-2]
;*
;*  +------->------+----- y[n]
;*  |       1      |
;*  |              |
;*  |              |
;*  |           +--+--+
;*  |           |  -1 |
;*  |           | z   |
;*  |           +--+--+
;*  |              |
;*  +-------<------+
;*  |    2cos(f)   |
;*  |           +--+--+
;*  |           |  -1 |
;*  |           | z   |
;*  |           +--+--+
;*  |      -1      |
;*  +-------<------+
;*
;*************************************************************************
SineGen:        ; Generate a Sine Wave
        SETC OVM
        SPM  1
        MAR  *+

SineLoop:       ; Calculate next sine value
        LACC *-,15
        NEG
        LT   *
        DMOV *
        MPY  sineFreq
        APAC
        SACH *,1
        LT   *+,AR1
        MPY  sineAmp
        PAC
        SACH *+,1,AR0
;       SACH *+,1,AR1
;       SACH *+,1,AR0
        BANZ SineLoop,*-,AR2

        SPM  0
SineExit
        RET

;*************************************************************************
;*  Modulator Subroutine
;*  --------------------
;*  Input: AR0 = number of samples - 1
;*         AR1 = start of input/output buffer
;*             AR2 = start of sine wave input buffer
;*         ARP = AR2
;*
;*  y[n] = (y[n] + alpha) * x[n]
;*
;*************************************************************************
Modlator:      ; Modulate input signal with sine wave
ModLoop         ; Loop for modulating each input sample
        LACL #0
        LT   *+,AR1
        MPY  alpha
        MPYA *
        APAC
        SACH *+,1,AR0
        BANZ ModLoop,*-,AR2
ModExit         ; End of Modlator Subroutine
        RET

Rectify       ; Rectify signal
;*************************************************************************
;*  Rectifier Subroutine
;*  --------------------
;*  Input: AR0 = number of samples - 1
;*         AR1 = start of input/output buffer
;*         ARP = AR1
;*
;*  y[n] = abs(y[n])
;*
;*************************************************************************
RectLoop           ; Loop for modulating each input sample
        LACC *
        ABS
        SACL *+,0,AR0
        BANZ RectLoop,*-,AR1
RectExit           ; End of Rectifier Subroutine
        RET

;*************************************************************************
;*  PeaknLeak Subroutine
;*  --------------------
;*  Input: AR0 = number of samples - 1
;*         AR1 = start of input/output buffer
;*         ARP = AR1
;*
;*  y[n] = (x[n] > y[n-1]) ? x[n] : y[n-1] * beta
;*
;*************************************************************************
PeakLeak ; Track the envelope of the input
        LT       beta
PeakLoop               ; Loop for envelope tracking
        LACC *
        SUB  lastPeak
        BCND  PeakNext, GT
    	MPY  lastPeak
    	PAC
        SACH *,1
PeakNext
        LACC *+,0,AR0
        SACL lastPeak
        BANZ PeakLoop,*-,AR1
PeakExit               ; End of PeaknLeak Subroutine
        RET

;*************************************************************************
;*  FIR Subroutine
;*  --------------------
;*  Input: AR0 = number of samples - 1
;*         AR1 = start of input/output buffer
;*         AR2 = delay line
;*         ARP = AR1
;*
;*************************************************************************
FIR   ; FIR filter subroutine
        SPM  3

FIRLoop         ; Filter Each Sample
        LACC     *,0,AR2
        SACL *
;       CONF 2
        ADRK 30
        LACL #0
        MPY     #0
        RPT     #30
        MACD pLPFCoef,*-
        MPYA *+,AR1
;       CONF 1
        SACH *+,7,AR0
        BANZ FIRLoop,*-,AR1
FIRExit         ; End of FIR Filter Subroutine
        SPM  0
        RET

;*************************************************************************
;*  Calculate Length of Interpolated Signal Buffer
;*  ----------------------------------------------
;*  Input: interFct = interpolation factor - 2
;*  Output:
;*         intBufSz = bufSize * (interFct + 2)
;*
;*************************************************************************
CalBufLn:
        LACC interFct
        ADD     #2
        SACL temp
        LT   temp
        MPY  bufSize
        PAC
        SACL intBufSz
        RET

;*************************************************************************
;*  Interpolate Subroutine
;*  ----------------------
;*  Input: AR0 = number of samples - 1
;*         AR1 = input/output buffer
;*         ARP = AR1
;*
;*************************************************************************
Intplate:
        SAR  AR1,temp
        LACC temp
        ADD  intBufSz
        SUB     #1
        SACL temp
        LAR  AR2,temp
        MAR  *0+

InterLp:
        LACL *-,AR2
        RPT  interFct
        SACH *-
        SACL *-,0,AR0
        BANZ InterLp,*-,AR1

        LAR  AR0,intBufSz
        LAR  AR1,#sigBuf
        LAR  AR2,#intDlyLn
        MAR  *,AR0
IntFilt:
        CALL FIR,*-,AR1
IntExit:
        RET

;;*************************************************************************
;;*  Decimate Subroutine
;;*  -------------------
;;*  Input: AR0 = number of samples - 1
;;*         AR1 = start of input/output buffer
;;*         ARP = AR1
;;*
;;*************************************************************************
;Decimate:
;       SAR  AR1,temp
;       LAR  AR2,temp
;       LARP AR1
;DecimateLoop:
;       ZALS *+,AR2
;       SACL *+,0,AR1
;       RPT  interFct
;       ZALS *+
;       LARP AR0
;       BANZ DecimateLoop,*-,AR1
;
;DecimateExit:
;       RET

;*************************************************************************
;*  DC removal Subroutine
;*  ---------------------
;*  Input: AR0 = number of samples - 1
;*         AR1 = start of input/output buffer
;*         ARP = AR1
;*
;*************************************************************************
RemoveDC:
        LACL #0
AvrgLoop:
        ADD  *+,1,AR0
        BANZ AvrgLoop,*-,AR1
        SACH dc,7

        LAR  AR1,#sigBuf
        LAR  AR0,intBufSz
        MAR *,AR0
        MAR  *-,AR1
SubDClp:
        LACC *
        SUB  dc
        SACL *+,0,AR0
        BANZ SubDClp,*-,AR1

        RET

