*******************************************************************************
*       (C) COPYRIGHT CURTIN UNIVERSITY, PERTH, WESTERN AUSTRALIA.            *   
*       Written by Ivar Siewert 930609A (February to June 1998).              *
*******************************************************************************
*                                                                             *
*  PROGRAM NAME: AUSTRALIAN CURRENCY NOTE READER FOR THE VISION IMPAIRED      *
*                                                                             *
*  DESCRIPTION: This program for the Texas Instruments TMS320C50 Digital      *
*               Signal Processor (DSP) is designed to recognise the           *
*               image of the clear window on Australian currency notes.       *
*               The image is produced by a Texas Instruments TSL218           *
*               linear array with a length of 512 pixels.                     *
*                                                                             *
*******************************************************************************

            .mmregs                 ;define global symbolic names for
                                    ; TMS320C5x registers.

*******************************************************************************
*                             GLOBAL CONSTANTS                                *
*******************************************************************************

DATA_PORT   .set  0050h             ;memory-mapped data Port Address 0.

*******************************************************************************
*                             GLOBAL VARIABLES                                *
*******************************************************************************

                  .ds         2b80h
LINE_BUFFER       .space      200h        ;storage for 512 pixels.
                  .ds         2ba0h
COMP_BUFFER       .space      200h
                  .ds         2bc0h
BUFFER            .word       00ffh
                  .ds         2bc1h
FLAGS             .word       0

*******************************************************************************
*                             INTERRUPT VECTORS                               *
*******************************************************************************

            ;set interrupt vectors

            .ps   0803h
            .word 0992h             ;store INT1 ISR address (CLKL signal).
            .ps   0802h
            .word 7980h             ;store branch instruction.
            .ps   0807h
            .word 0980h             ;store INT3 ISR address (SO signal).
            .ps   0806h
            .word 7980h             ;store branch instruction.
      
*******************************************************************************
*                         INTERRUPT SERVICE ROUTINES                          *
*******************************************************************************

            .ps   0980h             ;assemble code starting at this address.
            
            ;Serial Out SIGnal Interrupt Service Routine.
            ;This routine is triggered by external interrupt INT3.

SO_SIG_ISR  LAR   AR0,#2B80h
            LDP   #0
            RPT   #50
            OPL   #4000h,DATA_PORT  ;set DAT14, turn LEDs on.
            APL   #0BFFFh,DATA_PORT ;clear DAT14, turn LEDs off.
            LDP   #87
            RPT   #31
            APL   #0h,*+,AR0
            OPL   #01h,FLAGS        ;set SI flag for next scan.
            LDP   #0
            SPLK  #01h,IFR
            RETE

;------------------------------------------------------------------------------

		;CLocK Low Interrupt Service Routine

            ;This routine is triggered by external interrupt INT1.
            ;The SI flag is first tested to see if this is the beginning of
            ; a new scan. If so, the SI signal is set high, the flag is

CLKL_ISR    LACL  DATA_PORT         ;get pixel data from port.
            AND   #01h              ;mask out any extraneous bits.
            LDP   #87
            SACL  BUFFER            ;store pixel in buffer.
            BIT   FLAGS,15          ;test SI flag (bit 0).
            BCND  SI_SIGNAL,TC      ;set SI signal if flag clear,
                                    ; otherwise store pixel.
STORE_PIXEL LAMM  AR0               ;load current pixel counter.
            SAMM  TREG1             ;store bit position.      
            MAR   *-,AR1            ;decrement pixel counter, change ARP to 1.
            BSAR  4                 ;divide by 16, ACCL now holds index of
                                    ; a word in line buffer. TREG1 holds the
                                    ; bit storage position within that word. 
            SUB   #31               ;effectively reverse storage order of
            NEG                     ; words, 0 becomes 31, 31 becomes 0.
            EXAR                    ;swap into ACCB.
            LACC  #LINE_BUFFER      ;load start address of line buffer.
            ORB                     ;OR to yield complete pointer in ACCL.
            SAMM  AR1               ;save pointer in AR1.
            LACT  BUFFER            ;reload pel shifted into correct position.
            LDP   #0
            OR    *,AR1             ;OR word in line buffer and pixel in ACCL.
            SACL  *,0,AR0           ;store back into line buffer.
            RETE

SI_SIGNAL   APL   #0fffeh,FLAGS     ;clear SI flag.
            LDP   #0
            LAR   AR0,#511          ;reset pixel counter.
            OPL   #08000h,DATA_PORT ;set DAT15 high (SI signal).
HERE        BIT   IFR,15            
            BCND  HERE,NTC          ;wait for next INT1 interrupt.
            RPT   #10               ;wait a bit longer.
            NOP
            SPLK  #01h,IFR          ;clear the interrupt which occurred.
            APL   #7FFFh,DATA_PORT  ;clear DAT15 (SI signal).
            RETE

******************************************************************************
*                             SUBROUTINES                                    *
******************************************************************************

            ;TMS320C50 Initialisation.

            ;CWSR = Wait-State Control Register
            ;PDWSR = Program Data Wait-State Register
            ;IOWSR = Input Output Wait-State Register
            ;PMST = Processor Mode STatus register
            ;PRD = timer PeRioD register
            ;TCR = Timer Control Register
            ;IFR = Interrupt Flag Register
            ;IMR = Interupt Mask Register

INITIALISE  LDP   #0                ;data pointer points to page 0.
            CLRC  SXM               ;clear sign extension mode bit.
            SPLK  #0,CWSR           ;configure wait states.
            SPLK  #0,IOWSR          ;zero I/O wait states.
            SPLK  #0,PDWSR          ;zero program/data wait states.
            SPLK  #10h,TCR          ;timer stopped with no prescale.
            SPLK  #19,PRD           ;load timer for 1 us period (20MHz/[19+1]).
            OPL   #05h,IMR          ;unmask INT1 and INT3 external interrupts.
            SPLK  #0ffffh,IFR       ;clear any pending interrupts.
            CLRC  INTM              ;enable maskable interrupts.
            RET

;------------------------------------------------------------------------------

            ;explanation here

START_CCD   LDP   #87
            OPL   #01h,FLAGS        ;set SI flag (bit 0) to start scan.
            LDP   #0
            SPLK  #20h,TCR          ;reload and start timer.
                                    ;at least 12 cycles to purge pixels.
                                    ; then auto-calibrate.
            RET

;------------------------------------------------------------------------------

            ;explanation here

STOP_CCD    LDP   #0
            SPLK  #10h,TCR          ;stop timer immediately.
            RET

;------------------------------------------------------------------------------

            ;explanation here

THERE       LAR   AR0,#LINE_BUFFER
            LAR   AR1,#COMP_BUFFER
            LACC  #0h
            LDP   #87
            RPT   #31
            APL   #0h,*+,AR0        ;clear buffer
            MAR   *-,AR0
            LACC  #0f00ch
            SACL  *,0,AR1
            SACL  *,0,AR0
            LAR   AR0,#LINE_BUFFER
            SACL  *+,0,AR0
            SACL  *,0,AR0
            B     CRUD

CORRELATE   MAR   *,AR0             ;use AR0.
            ;LAR   AR4,511           ;rotate pixel buffer 512 times.
            LAR   AR0,#LINE_BUFFER  ;load start address of line buffer.
            ;LAR   AR1,#COMP_BUFFER  ;load start address of compare buffer.
            LACL  #15               ;repeat next block 16 times.
            SAMM  BRCR              ;load ACCL in block repeat register.
            CLRC  C                 ;clear any previous carry.

            RPTB  BLOCK1
            LACC  *+,16,AR0         ;load word into ACCH, point to next word.
            EXAR                    ;swap into ACCB.
            LACL  *-,AR0            ;next word into ACCL, reset to first word.
            ORB
            ROR                     ;shift right, with previous carry in MSB.
            SACH  *+,0,AR0          ;store first word back into line buffer.
BLOCK1      SACL  *+,0,AR0          ;store second word back into line buffer.

            ZAP                     ;clear accumulator.
            LAR   AR0,#LINE_BUFFER  ;load address of first word.
            ROR                     ;put carry bit from last word in ACCH.
            BSAR  15                ;shift right into MSB of ACCL.
            BSAR  1
            OR    *                 ;OR into ACCL.
            SACL  *                 ;save first word back into buffer.

CRUD        LAR   AR1,#COMP_BUFFER  ;load start address of line buffer.
            LACL  #31               ;repeat next block 32 times.
            SAMM  BRCR              ;load BRCR with buffer word count.
            LAR   AR2,#0h           ;clear error pixel counter.

            RPTB  BLOCK2
            MAR   *,AR0             ;change ARP to AR0.
            LAR   AR3,#16           ;load bit counter.
            LACL  *+,AR1            ;load word from line buffer, change ARP.
            EXAR                    ;swap into ACCB.
            LACL  *+,AR2            ;load word from compare buffer, new ARP.
            XORB                    ;XOR ACC and ACCB, result in ACCL.

COUNT       SFR                     ;shift 1 bit right.
            XC    0,C               ;test for carry bit, if it's a 1,
            MAR   *+                ; increment error pixel counter.
            MAR   *,AR3             ;change ARP to bit counter.   
            BANZ  COUNT,*-,AR2      ;count errors for all 16 bits in ACCL.
BLOCK2      NOP
            MAR   *,AR4
            ;BANZ  CORRELATE         ;branch if all 32 words are not done.

lop         B     lop
*******************************************************************************
*                             MAIN PROGRAM                                    *
*******************************************************************************

            ;explanation of main program

MAIN        B     THERE
            CALL  INITIALISE
            CALL  START_CCD
            NOP
            NOP
BACK        BCND  BACK,UNC           ;go back and wait for another.
            NOP
            NOP

            .entry MAIN
            .end

;------------------------------------------------------------------------------


