;                               BANDPASS FILTER
;
;  -> FIR FILTER
;     This program uses an 81 tap FIR filter to implement a BANDPASS filter.
;     The filter passes frequencies between 3KHz - 4KHz. Blackman filter
;     windowing was used when calculating the filter's coefficients.
;
;  -> AIC
;     AIC set-up parameters. For DSK running at 40 mhz, the sampling
;     frequency is 10,000,000/2/TA/TB for transmit and is
;     10,000,000/2/RA/RB for receive. The max value for TA,TB.RA,RB is 63
;     since the registers are 6 bits.
;
;     RANDOM NOISE GENERATOR
;  -> A random noise generator is included in this code to allow the user
;     to hear the differences in the filtered output or see them using a
;     spectrum analyzer. The random noise generator can be bipassed by placing
;     a semicolon ; at the leftmost column of every asm staement of the Random
;     Noise Generator.
;
;
;
;
;
	 .MMREGS
	 .ds      0f00h
TA       .word    16         ;
RA       .word    16         ;  This set up of AIC registers give
			     ;  a sampling freq of 10,081 Hz
			     ;
TB       .word    31         ;
RB       .word    31         ;
AIC_CTR  .word    08h
OUTPUT   .word    0
TEMP     .word    0               ; location of TEMPorary storage
TEMP1    .word    0               ;
seed     .word    07e6dh          ; seed for random noise generator

;Filter Coefficient Generator   bandpass @ flcut=3K  fhcut= 4K

h0      .word         0    ;  40        0.0000
h1      .word         0    ;  39        0.0000
h2      .word         1    ;  38        0.0001
h3      .word        -6    ;  37       -0.0004
h4      .word         5    ;  36        0.0003
h5      .word        22    ;  35        0.0013
h6      .word       -50    ;  34       -0.0031
h7      .word        13    ;  33        0.0008
h8      .word       100    ;  32        0.0061
h9      .word      -153    ;  31       -0.0093
h10     .word         0    ;  30       -0.0000
h11     .word       259    ;  29        0.0158
h12     .word      -287    ;  28       -0.0175
h13     .word       -65    ;  27       -0.0040
h14     .word       445    ;  26        0.0271
h15     .word      -354    ;  25       -0.0216
h16     .word      -156    ;  24       -0.0096
h17     .word       466    ;  23        0.0284
h18     .word      -226    ;  22       -0.0138
h19     .word      -105    ;  21       -0.0064
h20     .word         0    ;  20        0.0000
h21     .word       146    ;  19        0.0089
h22     .word       439    ;  18        0.0268
h23     .word     -1264    ;  17       -0.0771
h24     .word       596    ;  16        0.0364
h25     .word      1904    ;  15        0.1162
h26     .word     -3394    ;  14       -0.2072
h27     .word       712    ;  13        0.0434
h28     .word      4549    ;  12        0.2777
h29     .word     -6030    ;  11       -0.3681
h30     .word         0    ;  10       -0.0000
h31     .word      8181    ;   9        0.4993
h32     .word     -8408    ;   8       -0.5132
h33     .word     -1810    ;   7       -0.1105
h34     .word     12047    ;   6        0.7353
h35     .word     -9661    ;   5       -0.5897
h36     .word     -4489    ;   4       -0.2740
h37     .word     15040    ;   3        0.9180
h38     .word     -9255    ;   2       -0.5649
h39     .word     -7293    ;   1       -0.4451
Czero   .word     18688    ;   0         0.1000
h40     .word     -7293    ;   1       -0.4451
h41     .word     -9255    ;   2       -0.5649
h42     .word     15040    ;   3        0.9180
h43     .word     -4489    ;   4       -0.2740
h44     .word     -9661    ;   5       -0.5897
h45     .word     12047    ;   6        0.7353
h46     .word     -1810    ;   7       -0.1105
h47     .word     -8408    ;   8       -0.5132
h48     .word      8181    ;   9        0.4993
h49     .word         0    ;  10       -0.0000
h50     .word     -6030    ;  11       -0.3681
h51     .word      4549    ;  12        0.2777
h52     .word       712    ;  13        0.0434
h53     .word     -3394    ;  14       -0.2072
h54     .word      1904    ;  15        0.1162
h55     .word       596    ;  16        0.0364
h56     .word     -1264    ;  17       -0.0771
h57     .word       439    ;  18        0.0268
h58     .word       146    ;  19        0.0089
h59     .word         0    ;  20        0.0000
h60     .word      -105    ;  21       -0.0064
h61     .word      -226    ;  22       -0.0138
h62     .word       466    ;  23        0.0284
h63     .word      -156    ;  24       -0.0096
h64     .word      -354    ;  25       -0.0216
h65     .word       445    ;  26        0.0271
h66     .word       -65    ;  27       -0.0040
h67     .word      -287    ;  28       -0.0175
h68     .word       259    ;  29        0.0158
h69     .word         0    ;  30       -0.0000
h70     .word      -153    ;  31       -0.0093
h71     .word       100    ;  32        0.0061
h72     .word        13    ;  33        0.0008
h73     .word       -50    ;  34       -0.0031
h74     .word        22    ;  35        0.0013
h75     .word         5    ;  36        0.0003
h76     .word        -6    ;  37       -0.0004
h77     .word         1    ;  38        0.0001
h78     .word         0    ;  39        0.0000
h79     .word         0    ;  40        0.0000


XN       .word     0,0,0,0,0,0,0,0,0,0    ; 80 data locations for 80
XN1      .word     0,0,0,0,0,0,0,0,0,0    ; stage delay line
XN2      .word     0,0,0,0,0,0,0,0,0,0    ;
XN3      .word     0,0,0,0,0,0,0,0,0,0    ;
XN4      .word     0,0,0,0,0,0,0,0,0,0    ;
XN5      .word     0,0,0,0,0,0,0,0,0,0    ;
XN6      .word     0,0,0,0,0,0,0,0,0,0    ;
XN7      .word     0,0,0,0,0,0,0,0,0,0    ;
XNLAST   .word     0;

	.ps      0080ah
rint:    B       RECEIVE
xint:    B       TRANSMIT

	.ps       0a00h
	.entry                  ;
	;--------------------------------------------------------------

	SETC    INTM
	LDP     #0
	OPL     #0834h,PMST
	LACC    #0
	SAMM    CWSR
	SAMM    PDWSR
	SETC    SXM          ;  SXM MUST BE SET
	SPLK    #022h,IMR    ; This turns on receive interrupt only

	CALL    AICINIT      ;
	SPLK    #12h,IMR
	CLRC    OVM
	SPM     0            ;
	CLRC    INTM         ;

WAIT:                        ; a main program would go here
	B       WAIT         ;

RECEIVE:
	LDP      #XN
	CLRC     INTM        ; ENABLE INTERRUPTS FOR DEBUGGING DSK PURPOSES

;---- Random Noise Generator ------------------------
;        LACC    seed,1       ;*
;        XOR     seed         ;*
;        SACL    TEMP,2       ;*
;        XOR     TEMP ;       ;*
;        AND     #8000h       ;*
;        ADD     seed,16      ;*
;        SACH    seed,1       ;* Reduce the output by at least 1/8
;        LACC    seed,11      ;* to prevent the overflow
;        AND     #0FFFCh,15   ;*
;        RPT     #14          ;* When NOT using the random noise generator, be
;        SFR                  ;* sure to comment this section out using ;'s
;
;---- End of noise generator ------------------------

			     ; When not using noise generator, the next line
			     ; should NOT be commented out.
	LAMM    DRR          ; load accumulator with word received from AIC
	SACL     XN          ; store the value of received word to a variable

	LAR      AR0,#XNLAST ; load AR0 with address of last delay element
	ZAP                  ; ZERO ACC AND PRODUCT REGISTERS
	MAR      *,AR0       ; AR0 is current AR register

	RPT      #80         ; repeat the next line 81 times through the
	MACD     #h0,*-      ; complete coeff table.
	APAC                 ; Accumulate last product

	SACH     OUTPUT,1    ; ACC -> OUTPUT  Get rid of xtra sign bit.
	LACC     OUTPUT      ; OUTPUT -> ACC
	SFL                  ; OUTPUT*2
	AND      #0fffch     ; Last two must be zero for AIC
	SAMM     DXR         ; write output word to transmit register
	RETE                 ;


TRANSMIT:
	RETE


******************************************************************
;  DESCRIPTION: This routine initializes the 'C50 serial port    *
;               and the TLC320C40's (AIC) TA,RA,TB,RB and        *
;               control registers                                *
;*****************************************************************
;
AICINIT: SPLK    #20h,TCR           ; To generate 10 MHz from Tout
	 SPLK    #01h,PRD           ; for AIC master clock
	 MAR     *,AR0
	 LACC    #0008h             ; Non continuous mode
	 SACL    SPC                ; FSX as input
	 LACC    #00c8h             ; 16 bit words
	 SACL    SPC
	 LACC    #080h            ; Pulse AIC reset by setting it low
	 SACH    DXR
	 SACL    GREG
	 LAR     AR0,#0FFFFh
	 RPT     #10000           ; and taking it high after 10000 cycles
	 LACC    *,0,AR0          ; (.5ms at 50ns)
	 SACH    GREG
	 ;------------------------
	 LDP     #TA              ;
	 SETC    SXM              ;
	 LACC    TA,9             ; Initialize TA and RA register
	 ADD     RA,2             ;
	 CALL    AIC_2ND          ;
	 ;------------------------
	 LDP     #TB
	 LACC    TB,9             ; Initialize TB and RB register
	 ADD     RB,2             ;
	 ADD     #02h             ;
	 CALL    AIC_2ND          ;
	 ;------------------------
	 LDP     #AIC_CTR
	 LACC    AIC_CTR,2        ; Initialize control register
	 ADD     #03h             ;
	 CALL    AIC_2ND          ;
	 RET                      ;

AIC_2ND:
	 LDP     #0               ; Data page point is 0  (MM regs)
	 SACH    DXR              ; send ACChi 00
	 CLRC    INTM             ; enable interrupts
	 IDLE                     ; wait for interrupt
	 ADD     #6h,15           ; 0000 0000 0000 0011 XXXX XXXX XXXX XXXX b
	 SACH    DXR              ; send ACChi to initiate secondary protocol
	 IDLE                     ; wait for interrupt
	 SACL    DXR              ; send the T register data
	 IDLE                     ; wait for interrupt
	 LACL    #0               ; clear ACClo
	 SACL    DXR              ; send another to make sure 1st word got sent
	 IDLE                     ; wait for interrupt
	 SETC    INTM
	 RET                      ;
	.END

