;                               BANDSTOP FILTER
;
;  -> FIR FILTER
;     This program uses an 81 tap FIR filter to implement a BANDSTOP filter.
;     The filter stops frequencies between 1KHz - 3.5KHz. Blackman filter
;     windowing was used when calculating the filter's coefficients.
;
;  -> AIC
;     AIC set-up parameters. For DSK running at 40 mhz, the sampling
;     frequency is 10,000,000/2/TA/TB for transmit and is
;     10,000,000/2/RA/RB for receive. The max value for TA,TB.RA,RB is 63
;     since the registers are 6 bits.
;
;     RANDOM NOISE GENERATOR
;  -> A random noise generator is included in this code to allow the user
;     to hear the differences in the filtered output or see them using a
;     spectrum analyzer. The random noise generator can be bipassed by placing
;     a semicolon ; at the leftmost column of every asm staement of the Random
;     Noise Generator.
;
;
;
;
; ------------------------------------------------------------------------

         .MMREGS
         .ds      0f00h
TA       .word    16         ;
RA       .word    16         ;  This set up of AIC registers give
                             ;  a sampling freq of 10,081 Hz
                             ;
TB       .word    31         ;
RB       .word    31         ;
AIC_CTR  .word    08h
OUTPUT   .word    0
TEMP     .word    0               ; location of TEMPorary storage
TEMP1    .word    0               ;
seed     .word    07e6dh          ; seed for random noise generator

;Filter Coefficient Generator   bandstop flcut=1K  fhcut = 3.5K
h0      .word         0    ;  40       -0.0000
h1	.word         0    ;  39        0.0000
h2      .word        -3    ;  38       -0.0004
h3      .word        -2    ;  37       -0.0003
h4	.word         0    ;  36        0.0000
h5      .word       -11    ;  35       -0.0013
h6	.word        19    ;  34        0.0023
h7	.word        29    ;  33        0.0036
h8      .word         0    ;  32       -0.0000
h9	.word        61    ;  31        0.0074
h10	.word         0    ;  30        0.0000
h11     .word      -103    ;  29       -0.0125
h12     .word         0    ;  28       -0.0000
h13     .word      -147    ;  27       -0.0179
h14     .word      -170    ;  26       -0.0207
h15	.word       177    ;  25        0.0216
h16	.word         0    ;  24        0.0000
h17	.word       167    ;  23        0.0203
h18	.word       591    ;  22        0.0722
h19     .word       -82    ;  21       -0.0100
h20     .word         0    ;  20       -0.0000
h21	.word       114    ;  19        0.0139
h22     .word     -1148    ;  18       -0.1401
h23     .word      -452    ;  17       -0.0552
h24	.word         0    ;  16        0.0000
h25     .word      -952    ;  15       -0.1162
h26	.word      1296    ;  14        0.1583
h27	.word      1609    ;  13        0.1964
h28     .word         0    ;  12       -0.0000
h29	.word      2393    ;  11        0.2921
h30	.word         0    ;  10        0.0000
h31     .word     -3246    ;   9       -0.3963
h32     .word         0    ;   8       -0.0000
h33     .word     -4089    ;   7       -0.4991
h34     .word     -4601    ;   6       -0.5617
h35	.word      4831    ;   5        0.5897
h36	.word         0    ;   4        0.0000
h37	.word      5384    ;   3        0.6572
h38	.word     24230    ;   2        2.9578
h39     .word     -5679    ;   1       -0.6933
Czero   .word     32767
h40     .word     -5679    ;   1       -0.6933
h41	.word     24230    ;   2        2.9578
h42	.word      5384    ;   3        0.6572
h43	.word         0    ;   4        0.0000
h44	.word      4831    ;   5        0.5897
h45     .word     -4601    ;   6       -0.5617
h46     .word     -4089    ;   7       -0.4991
h47     .word         0    ;   8       -0.0000
h48     .word     -3246    ;   9       -0.3963
h49	.word         0    ;  10        0.0000
h50	.word      2393    ;  11        0.2921
h51     .word         0    ;  12       -0.0000
h52	.word      1609    ;  13        0.1964
h53	.word      1296    ;  14        0.1583
h54     .word      -952    ;  15       -0.1162
h55	.word         0    ;  16        0.0000
h56     .word      -452    ;  17       -0.0552
h57     .word     -1148    ;  18       -0.1401
h58	.word       114    ;  19        0.0139
h59     .word         0    ;  20       -0.0000
h60     .word       -82    ;  21       -0.0100
h61     .word       591    ;  22        0.0722
h62	.word       167    ;  23        0.0203
h63	.word         0    ;  24        0.0000
h64	.word       177    ;  25        0.0216
h65     .word      -170    ;  26       -0.0207
h66     .word      -147    ;  27       -0.0179
h67     .word        -0    ;  28       -0.0000
h68     .word      -103    ;  29       -0.0125
h69	.word         0    ;  30        0.0000
h70	.word        61    ;  31        0.0074
h71     .word         0    ;  32       -0.0000
h72	.word        29    ;  33        0.0036
h73	.word        19    ;  34        0.0023
h74     .word       -11    ;  35       -0.0013
h75	.word         0    ;  36        0.0000
h76     .word        -2    ;  37       -0.0003
h77     .word        -3    ;  38       -0.0004
h78	.word         0    ;  39        0.0000
h79     .word         0    ;  40       -0.0000


XN       .word     0,0,0,0,0,0,0,0,0,0    ; 80 data locations for 80
XN1      .word     0,0,0,0,0,0,0,0,0,0    ; stage delay line
XN2      .word     0,0,0,0,0,0,0,0,0,0    ;
XN3      .word     0,0,0,0,0,0,0,0,0,0    ;
XN4      .word     0,0,0,0,0,0,0,0,0,0    ;
XN5      .word     0,0,0,0,0,0,0,0,0,0    ;
XN6      .word     0,0,0,0,0,0,0,0,0,0    ;
XN7      .word     0,0,0,0,0,0,0,0,0,0    ;
XNLAST   .word     0;

        .ps      0080ah
rint:    B       RECEIVE
xint:    B       TRANSMIT

        .ps       0a00h
        .entry                  ;
        ;--------------------------------------------------------------

        SETC    INTM
        LDP     #0
        OPL     #0834h,PMST
        LACC    #0
        SAMM    CWSR
        SAMM    PDWSR
        SETC    SXM           ;  SXM MUST BE SET
        SPLK    #022h,IMR     ; This turns on receive interrupt only

        CALL    AICINIT       ;
        SPLK    #12h,IMR
        CLRC    OVM
        SPM     0             ;
        CLRC    INTM          ;

WAIT:                         ; a main program would go here
        B       WAIT          ;

RECEIVE:
        LDP      #XN
        CLRC     INTM        ; ENABLE INTERRUPTS FOR DEBUGGING DSK PURPOSES

;----- Random Noise Generator --------------------
        LACC    seed,1       ;*
        XOR     seed         ;*
        SACL    TEMP,2       ;*
        XOR     TEMP ;       ;*
        AND     #8000h       ;*
        ADD     seed,16      ;*
        SACH    seed,1       ;* Reduce the output by at least 1/8
        LACC    seed,11      ;* to prevent the overflow
        AND     #0FFFCh,15   ;*
        RPT     #14          ;* When NOT using the generator, be sure this
        SFR                  ;* section is commented out using ;'s

; ----- End of Random Noise Generator -----------


                             ; The next line should not be commented out if
                             ; the generator code is commented out.
;       LAMM    DRR          ; load accumulator with word received from AIC
        SACL     XN          ; store the value of received word to a variable

        LAR      AR0,#XNLAST ; load AR0 with address of last delay element
        ZAP                  ; ZERO ACC AND PRODUCT REGISTERS
        MAR      *,AR0       ; AR0 is the current AR register.

        RPT      #80         ; Repeat the next line 81 times through the
        MACD     #h0,*-      ; complete coeff table.
        APAC                 ; Accumulate last procuct

        SACH     OUTPUT,1    ; ACC -> OUTPUT  Get rid of xtra sign bit
        LACC     OUTPUT
        SFL
        AND      #0fffch     ; The two LSB's must be zero fro AIC
        SAMM     DXR         ; write output word to transmit register
        RETE                 ;


TRANSMIT:
        RETE


******************************************************************
;  DESCRIPTION: This routine initializes the 'C50 serial port    *
;               and the TLC320C40's (AIC) TA,RA,TB,RB and        *
;               control registers                                *
;*****************************************************************
;
AICINIT: SPLK    #20h,TCR           ; To generate 10 MHz from Tout
         SPLK    #01h,PRD           ; for AIC master clock
         MAR     *,AR0
         LACC    #0008h             ; Non continuous mode
         SACL    SPC                ; FSX as input
         LACC    #00c8h             ; 16 bit words
         SACL    SPC
         LACC    #080h            ; Pulse AIC reset by setting it low
         SACH    DXR
         SACL    GREG
         LAR     AR0,#0FFFFh
         RPT     #10000           ; and taking it high after 10000 cycles
         LACC    *,0,AR0          ; (.5ms at 50ns)
         SACH    GREG
         ;------------------------
         LDP     #TA              ;
         SETC    SXM              ;
         LACC    TA,9             ; Initialize TA and RA register
         ADD     RA,2             ;
         CALL    AIC_2ND          ;
         ;------------------------
         LDP     #TB
         LACC    TB,9             ; Initialize TB and RB register
         ADD     RB,2             ;
         ADD     #02h             ;
         CALL    AIC_2ND          ;
         ;------------------------
         LDP     #AIC_CTR
         LACC    AIC_CTR,2        ; Initialize control register
         ADD     #03h             ;
         CALL    AIC_2ND          ;
         RET                      ;

AIC_2ND:
         LDP     #0               ; Data page point is 0  (MM regs)
         SACH    DXR              ; send ACChi 00
         CLRC    INTM             ; enable interrupts
         IDLE                     ; wait for interrupt
         ADD     #6h,15           ; 0000 0000 0000 0011 XXXX XXXX XXXX XXXX b
         SACH    DXR              ; send ACChi to initiate secondary protocol
         IDLE                     ; wait for interrupt
         SACL    DXR              ; send the T register data
         IDLE                     ; wait for interrupt
         LACL    #0               ; clear ACClo
         SACL    DXR              ; send another to make sure 1st word got sent
         IDLE                     ; wait for interrupt
         SETC    INTM
         RET                      ;
        .END

