;  TITLE       : DSPfilt.asm                            
;  Description : This program impliments a FIR filter. 
;                The coefficients determine the function,
;                i.e. bandpass,lowpass,highpass,bandstop
;                and are loaded from the file coeff.dat at
;                compilation. Coefficients are calculated
;                in WFilter by Computer Magic.
;  Author      : Iain Murray
;  Date        : 10 August 1998
;                
;  Specs       : This program impliments an 81 tap FIR filter on the TI 
;                TMS320C50 DSK. The Aic is initialised with a sample rate
;                of 10,081 Hz, anti aliasing bandpass filter enabled and 
;                unity gain. Filter pass band is 300 to 3.6kHz
;


         .MMREGS
         .ds      0f00h
TA       .word    16                      ;
RA       .word    16                      ;  This set up of AIC registers give
                                          ;  a sampling freq of 10,081 Hz
                                          ;
TB       .word    31                      ;
RB       .word    31                      ;
AIC_CTR  .word    08h                     ; AIC control word
OUTPUT   .word    0
TEMP     .word    0                       ; location of TEMPorary storage
TEMP1    .word    0                       ;


         .include "hpcoeff.asm"             ; Insert coefficient table


XN       .word     0,0,0,0,0,0,0,0,0,0    ; 80 data locations for 80
XN1      .word     0,0,0,0,0,0,0,0,0,0    ; stage delay line
XN2      .word     0,0,0,0,0,0,0,0,0,0    ;
XN3      .word     0,0,0,0,0,0,0,0,0,0    ;
XN4      .word     0,0,0,0,0,0,0,0,0,0    ;
XN5      .word     0,0,0,0,0,0,0,0,0,0    ;
XN6      .word     0,0,0,0,0,0,0,0,0,0    ;
XN7      .word     0,0,0,0,0,0,0,0,0,0    ;
XNLAST   .word     0                      ; And makes 81

        .ps      0080ah
rint:    B       RECEIVE
xint:    B       TRANSMIT

        .ps       0a00h
        .entry                  
        ;--------------------------------------------------------------

        SETC    INTM
        LDP     #0
        OPL     #0834h,PMST
        LACC    #0
        SAMM    CWSR         ; No wait states
        SAMM    PDWSR        ; As above
        SETC    SXM          ; SXM MUST BE SET
        SPLK    #022h,IMR    ; This turns on receive interrupt only

        CALL    AICINIT      ; Set up and init AIC
        SPLK    #12h,IMR     ; Enable XINT for transmission
        CLRC    OVM          ; Clear flag
        SPM     0            ; Set output scaler (no shift)
        CLRC    INTM         ; Enable interupts

WAIT:                        ; Wait for RINT input from AIC
        B       WAIT         ;

RECEIVE:
        LDP      #XN
        CLRC     INTM        ; ENABLE INTERRUPTS FOR DEBUGGING DSK PURPOSES
        LAMM     DRR         ; load accumulator with word received from AIC
        SACL     XN          ; store the value of received word to a variable
        LAR      AR0,#XNLAST ; load AR0 with address of last delay element
        ZAP                  ; ZERO ACC AND PRODUCT REGISTERS
        MAR      *,AR0       ; AR0 is current AR register

        RPT      #80         ; repeat the next line 81 times through the
        MACD     #h0,*-      ; complete coeff table.
        APAC                 ; Accumulate last product

        SACH     OUTPUT,1    ; ACC -> OUTPUT  Get rid of xtra sign bit.
        LACC     OUTPUT      ; OUTPUT -> ACC
        SFL                  ; OUTPUT*2
        AND      #0fffch     ; AIC uses last 2 bits for config
        SAMM     DXR         ; write output word to transmit register
        RETE                 ;


TRANSMIT:                    ; Unused. For data output only
        RETE


******************************************************************
;  DESCRIPTION: This routine initializes the 'C50 serial port    *
;               and the TLC320C40's (AIC) TA,RA,TB,RB and        *
;               control registers                                *
;*****************************************************************
;
AICINIT: SPLK    #20h,TCR           ; To generate 10 MHz from Tout
         SPLK    #01h,PRD           ; for AIC master clock
         MAR     *,AR0
         LACC    #0008h             ; Non continuous mode
         SACL    SPC                ; FSX as input
         LACC    #00c8h             ; 16 bit words
         SACL    SPC
         LACC    #080h              ; Pulse AIC reset by setting it low
         SACH    DXR
         SACL    GREG
         LAR     AR0,#0FFFFh
         RPT     #10000             ; and taking it high after 10000 cycles
         LACC    *,0,AR0            ; (.5ms at 50ns)
         SACH    GREG
         ;------------------------
         LDP     #TA              ;
         SETC    SXM              ;
         LACC    TA,9             ; Initialize TA and RA register
         ADD     RA,2             ;
         CALL    AIC_2ND          ;
         ;------------------------
         LDP     #TB
         LACC    TB,9             ; Initialize TB and RB register
         ADD     RB,2             ;
         ADD     #02h             ;
         CALL    AIC_2ND          ;
         ;------------------------
         LDP     #AIC_CTR
         LACC    AIC_CTR,2        ; Initialize control register
         ADD     #03h             ;
         CALL    AIC_2ND          ;
         RET                      ;

AIC_2ND:
         LDP     #0               ; Data page point is 0  (MM regs)
         SACH    DXR              ; send ACChi 00
         CLRC    INTM             ; enable interrupts
         IDLE                     ; wait for interrupt
         ADD     #6h,15           ; 0000 0000 0000 0011 XXXX XXXX XXXX XXXX b
         SACH    DXR              ; send ACChi to initiate secondary protocol
         IDLE                     ; wait for interrupt
         SACL    DXR              ; send the T register data
         IDLE                     ; wait for interrupt
         LACL    #0               ; clear ACC low
         SACL    DXR              ; send another to make sure 1st word got sent
         IDLE                     ; wait for interrupt
         SETC    INTM             ; disable interupts
         RET                      ;
        .END

