;                               HIGH-PASS FILTER
;
;  -> FIR FILTER
;     This program uses an 81 tap FIR filter to implement a HIPASS filter.
;     The filter passes frequencies greater than 3.0KHz. Blackman filter
;     windowing was used when calculating the filter's coefficients.
;
;  -> AIC
;     AIC set-up parameters. For DSK running at 40 mhz, the sampling
;     frequency is 10,000,000/2/TA/TB for transmit and is
;     10,000,000/2/RA/RB for receive. The max value for TA,TB.RA,RB is 63
;     since the registers are 6 bits.
;
;     RANDOM NOISE GENERATOR
;  -> A random noise generator is included in this code to allow the user
;     to hear the differences in the filtered output or see them using a
;     spectrum analyzer. The random noise generator can be bipassed by placing
;     a semicolon ; at the leftmost column of every asm staement of the Random
;     Noise Generator.
;
;
;
;
; ------------------------------------------------------------------------

         .MMREGS
         .ds      0f00h
TA       .word    16         ;
RA       .word    16         ;  This set up of AIC registers give
                             ;  a sampling freq of 10,081 Hz
                             ;
TB       .word    31         ;
RB       .word    31         ;
AIC_CTR  .word    08h
OUTPUT   .word    0
TEMP     .word    0               ; location of TEMPorary storage
TEMP1    .word    0               ;
seed     .word    07e6dh          ; seed for random noise generator

; Hipass filter flcut = 3000hz aprox.
h0      .word         0    ;  40       -0.0000
h1      .word         0    ;  39        0.0001
h2      .word        -1    ;  38       -0.0001
h3      .word        -3    ;  37       -0.0003
h4	.word         8    ;  36        0.0010
h5	.word         0    ;  35        0.0000
h6      .word       -19    ;  34       -0.0024
h7	.word        17    ;  33        0.0021
h8	.word        24    ;  32        0.0029
h9      .word       -52    ;  31       -0.0063
h10     .word         0    ;  30       -0.0000
h11	.word        87    ;  29        0.0107
h12     .word       -68    ;  28       -0.0084
h13     .word       -86    ;  27       -0.0105
h14	.word       172    ;  26        0.0210
h15	.word         0    ;  25        0.0000
h16     .word      -256    ;  24       -0.0312
h17	.word       191    ;  23        0.0233
h18	.word       228    ;  22        0.0279
h19     .word      -440    ;  21       -0.0537
h20     .word         0    ;  20       -0.0000
h21	.word       612    ;  19        0.0747
h22     .word      -443    ;  18       -0.0541
h23     .word      -518    ;  17       -0.0632
h24	.word       975    ;  16        0.1190
h25	.word         0    ;  15        0.0000
h26     .word     -1311    ;  14       -0.1601
h27	.word       938    ;  13        0.1145
h28	.word      1086    ;  12        0.1326
h29     .word     -2037    ;  11       -0.2486
h30     .word         0    ;  10       -0.0000
h31	.word      2763    ;   9        0.3373
h32     .word     -2007    ;   8       -0.2450
h33     .word     -2384    ;   7       -0.2911
h34	.word      4653    ;   6        0.5680
h35	.word         0    ;   5        0.0000
h36     .word     -7345    ;   4       -0.8966
h37	.word      6161    ;   3        0.7521
h38	.word      9359    ;   2        1.1425
h39     .word     -30518   ;   1       -3.7254
Czero   .word      32767
h40     .word     -30518   ;   1       -3.7254
h41	.word      9359    ;   2        1.1425
h42	.word      6161    ;   3        0.7521
h43     .word     -7345    ;   4       -0.8966
h44	.word         0    ;   5        0.0000
h45	.word      4653    ;   6        0.5680
h46     .word     -2384    ;   7       -0.2911
h47     .word     -2007    ;   8       -0.2450
h48	.word      2763    ;   9        0.3373
h49     .word         0    ;  10       -0.0000
h50     .word     -2037    ;  11       -0.2486
h51	.word      1086    ;  12        0.1326
h52	.word       938    ;  13        0.1145
h53     .word     -1311    ;  14       -0.1601
h54	.word         0    ;  15        0.0000
h55	.word       975    ;  16        0.1190
h56     .word      -518    ;  17       -0.0632
h57     .word      -443    ;  18       -0.0541
h58	.word       612    ;  19        0.0747
h59     .word         0    ;  20       -0.0000
h60     .word      -440    ;  21       -0.0537
h61	.word       228    ;  22        0.0279
h62	.word       191    ;  23        0.0233
h63     .word      -256    ;  24       -0.0312
h64	.word         0    ;  25        0.0000
h65	.word       172    ;  26        0.0210
h66     .word       -86    ;  27       -0.0105
h67     .word       -68    ;  28       -0.0084
h68	.word        87    ;  29        0.0107
h69     .word         0    ;  30       -0.0000
h70     .word       -52    ;  31       -0.0063
h71	.word        24    ;  32        0.0029
h72	.word        17    ;  33        0.0021
h73     .word       -19    ;  34       -0.0024
h74	.word         0    ;  35        0.0000
h75	.word         8    ;  36        0.0010
h76     .word        -3    ;  37       -0.0003
h77     .word        -1    ;  38       -0.0001
h78     .word         0    ;  39        0.0001
h79     .word         0    ;  40       -0.0000


XN       .word     0,0,0,0,0,0,0,0,0,0    ; 80 data locations for 80
XN1      .word     0,0,0,0,0,0,0,0,0,0    ; stage delay line
XN2      .word     0,0,0,0,0,0,0,0,0,0    ;
XN3      .word     0,0,0,0,0,0,0,0,0,0    ;
XN4      .word     0,0,0,0,0,0,0,0,0,0    ;
XN5      .word     0,0,0,0,0,0,0,0,0,0    ;
XN6      .word     0,0,0,0,0,0,0,0,0,0    ;
XN7      .word     0,0,0,0,0,0,0,0,0,0    ;
XNLAST   .word     0;

        .ps      0080ah
rint:    B       RECEIVE
xint:    B       TRANSMIT

        .ps       0a00h
        .entry                  ;
        ;--------------------------------------------------------------

        SETC    INTM
        LDP     #0
        OPL     #0834h,PMST
        LACC    #0
        SAMM    CWSR
        SAMM    PDWSR
        SETC    SXM           ;  SXM MUST BE SET
        SPLK    #022h,IMR     ; This turns on receive interrupt only

        CALL    AICINIT       ;
        SPLK    #12h,IMR
        CLRC    OVM
        SPM     1             ;
        CLRC    INTM          ;

WAIT:                         ; a main program would go here
        B       WAIT          ;

RECEIVE:
        LDP      #XN
        CLRC     INTM        ; ENABLE INTERRUPTS FOR DEBUGGING DSK PURPOSES

; ----- Random Noise Generator -----------------
        LACC    seed,1       ;
        XOR     seed         ;
        SACL    TEMP,2       ;
        XOR     TEMP ;       ;
        AND     #8000h       ;
        ADD     seed,16      ;
        SACH    seed,1       ; Reduce the output by at least 1/8
        LACC    seed,11      ; to prevent the overflow
        AND     #0FFFCh,15   ;
        RPT     #14          ;
        SFR                  ;

; ----- End of Noise Generator -----------------

                             ; The next line should not be commented out if you
                             ; are the generator code above is commented out.
;       LAMM DRR             ; load accumulator with word received from AIC
        SACL     XN          ; store the value of received word to a variable

        LAR      AR0,#XNLAST ; load AR0 with address of last delay element
        ZAP                  ; ZERO ACC AND PRODUCT REGISTERS
        MAR      *,AR0       ; AR0 is the current AR register

        RPT      #80         ; Repeat the next instruction 81 times through the
        MACD     #h0 ,*-     ; complete coeff table.
        APAC                 ; Accumulate last product.

        SACH     OUTPUT,1    ; ACC -> OUTPUT  Get rid of xtra sign bit.
        LACC     OUTPUT      ;
        SFL
        AND      #0fffch     ; Two LSB's must be zero for AIC
        SAMM     DXR         ; write output word to transmit register
        RETE                 ;


TRANSMIT:
        RETE


******************************************************************
;  DESCRIPTION: This routine initializes the 'C50 serial port    *
;               and the TLC320C40's (AIC) TA,RA,TB,RB and        *
;               control registers                                *
;*****************************************************************
;
AICINIT: SPLK    #20h,TCR           ; To generate 10 MHz from Tout
         SPLK    #01h,PRD           ; for AIC master clock
         MAR     *,AR0
         LACC    #0008h             ; Non continuous mode
         SACL    SPC                ; FSX as input
         LACC    #00c8h             ; 16 bit words
         SACL    SPC
         LACC    #080h            ; Pulse AIC reset by setting it low
         SACH    DXR
         SACL    GREG
         LAR     AR0,#0FFFFh
         RPT     #10000           ; and taking it high after 10000 cycles
         LACC    *,0,AR0          ; (.5ms at 50ns)
         SACH    GREG
         ;------------------------
         LDP     #TA              ;
         SETC    SXM              ;
         LACC    TA,9             ; Initialize TA and RA register
         ADD     RA,2             ;
         CALL    AIC_2ND          ;
         ;------------------------
         LDP     #TB
         LACC    TB,9             ; Initialize TB and RB register
         ADD     RB,2             ;
         ADD     #02h             ;
         CALL    AIC_2ND          ;
         ;------------------------
         LDP     #AIC_CTR
         LACC    AIC_CTR,2        ; Initialize control register
         ADD     #03h             ;
         CALL    AIC_2ND          ;
         RET                      ;

AIC_2ND:
         LDP     #0               ; Data page point is 0  (MM regs)
         SACH    DXR              ; send ACChi 00
         CLRC    INTM             ; enable interrupts
         IDLE                     ; wait for interrupt
         ADD     #6h,15           ; 0000 0000 0000 0011 XXXX XXXX XXXX XXXX b
         SACH    DXR              ; send ACChi to initiate secondary protocol
         IDLE                     ; wait for interrupt
         SACL    DXR              ; send the T register data
         IDLE                     ; wait for interrupt
         LACL    #0               ; clear ACClo
         SACL    DXR              ; send another to make sure 1st word got sent
         IDLE                     ; wait for interrupt
         SETC    INTM
         RET                      ;
        .END

