
            .mmregs                 ;define global symbolic names for
                                    ; TMS320C5x registers.

*******************************************************************************
*                             GLOBAL CONSTANTS                                *
*******************************************************************************

DATA_PORT   .set  0050h             ;memory-mapped data Port Address 0.

*******************************************************************************
*                             GLOBAL VARIABLES                                *
*******************************************************************************

                  .ds         2B80h
FLAGS             .word       0           ;Explanation of flags:
                                          ;bit 0      SI flag.

*******************************************************************************
*                             INTERRUPT VECTORS                               *
*******************************************************************************

            ;set interrupt vectors

            .ps   0803h
            .word 098Ah             ;store INT1 ISR address (CLKL signal).
            .ps   0802h
            .word 7980h             ;store branch instruction.
            .ps   0807h
            .word 0980h             ;store INT3 ISR address (SO signal).
            .ps   0806h
            .word 7980h             ;store branch instruction.
      
*******************************************************************************
*                         INTERRUPT SERVICE ROUTINES                          *
*******************************************************************************

            .ps   0980h             ;assemble code starting at this address.
            
            ;Serial Out SIGnal Interrupt Service Routine.
            ;This routine is triggered by external interrupt INT3.

SO_SIG_ISR  LDP   #0
            RPT   #22     
            OPL   #4000h,DATA_PORT  ;set DAT14, turn LEDs on.
            APL   #0BFFFh,DATA_PORT ;clear DAT14, turn LEDs off.
            LDP   #87
            OPL   #01h,FLAGS        ;set SI flag for next scan.
            RETE

;------------------------------------------------------------------------------

		;CLocK Low Interrupt Service Routine

            ;This routine is triggered by external interrupt INT1.
            ;The SI flag is first tested to see if this is the beginning of
            ; a new scan. If so, the SI signal is set high, the flag is
            ; cleared and the routine is exited. A zero SI flag means that
            ; valid pixel data needs to read and stored.
            ;The Analogue Out voltage is valid for a period of half a clock
            ; cycle immediately following a falling clock edge. This is
            ; when the analogue to digital conversion must take place.

CLKL_ISR    OPL    #0001h,DATA_PORT
	    ;LDP   #87
            ;BIT   FLAGS,15          ;test SI flag (bit 0).
            ;BCND  GET_PIXEL,NTC     ;set SI signal if flag set, otherwise
                                    ; get pixel and store.
            ;LDP   #0
            ;OPL   #8000h,DATA_PORT  ;set DAT15 high (SI signal).
            ;LDP   #87
            ;APL   #0FFFEh,FLAGS     ;clear SI flag.
            ;RETE

;GET_PIXEL   LDP   #0
            ;APL   #7FFFh,DATA_PORT  ;clear DAT15 (SI signal).
            ;LDP   #87
            ;IN    BUFFER,DATA_PORT  ;get data from port.
            ;APL   #1000h,BUFFER     ;extract pixel (bit 12).
            ;LACC  BUFFER            ;put in accumulator.
                                    ;store pixel.                  
            RETE

                      ;DON'T FORGET TO UPDATE INTERRUPT VECTORS.

******************************************************************************
*                             SUBROUTINES                                    *
******************************************************************************

            ;TMS320C50 Initialisation.

            ;PMST = Processor Mode STatus register
            ;PRD = timer PeRioD register
            ;TCR = Timer Control Register
            ;IFR = Interrupt Flag Register
            ;IMR = Interupt Mask Register

INITIALISE  LDP   #0                ;data pointer points to page 0.
            SPLK  #10h,TCR          ;timer stopped with no prescale.
            SPLK  #19,PRD           ;load timer for 1 us period (20MHz/[19+1]).
            OPL   #05h,IMR          ;unmask INT1 and INT3 external interrupts.
            CLRC  INTM              ;enable maskable interrupts.
            RET

            ;explanation here

START_CCD   LDP   #87
            OPL   #01h,FLAGS        ;set SI flag (bit 0) to start scan.
            LDP   #0
            SPLK  #20h,TCR          ;reload and start timer.
                                    ;at least 12 cycles to purge pixels.
                                    ; then auto-calibrate.
            RET

*******************************************************************************
*                             MAIN PROGRAM                                    *
*******************************************************************************

            ;explanation of main program

MAIN        CALL  INITIALISE
            CALL  START_CCD
HERE        B     HERE    		;wait for interrupts.
        
            .entry MAIN
            .end

;------------------------------------------------------------------------------

