	.mmregs
	.text
*
* This routine uses the BLDD instruction to move external data memory to
* internal data memory.
*
MOVEDD	   SPLK    #4000h,BMAR ; BMAR contains source address in data memory.
	   LAR	   AR7,#100h   ; AR7 contains destination address in data memory
	   MAR	   *,AR7       ; LARP = AR7.
	   RPT	   #1023       ; Move 1024 value to data memory blocks B0 and B1
	   BLDD    BMAR,*+
	   RET
*
* This routine uses the BLDP instruction to move external data memory to
* internal program memory.  This instruction could be used to boot load a
* program to the 8K on chip program memory from external data memory.
*
MOVEDP	   SPLK    #800H,BMAR  ; BMAR contains destination address in program
			       ; memory.
	   LAR	   AR7,#0E000h ; AR7 contains source address in data memory.
	   RPT	   #8191       ; Move 8k to program memory space.
	   BLDP    *+
	   RET
*
* This routine uses the BLPD instruction to move external program memory to
* internal data memory.  This routine is useful for loading a coefficient
* table stored in external program memory to data memory when no external
* data memory is available.
*
MOVEPD	   LAR	   AR7,#100h   ; AR7 points to destination in data memory.
	   RPT	   #127        ; Move 128 values from external program to
	   BLPD    #3800h,*+   ; internal data memory B0.
	   RET
*
* This routine uses the TBLR instruction to move program memory to data memory
* space.  This differs from the BLPD instruction in that the accumulator
* contains the address in program memory from which to transfer.  This allows
* for a calculated, rather than pre-determined, location in program memory to
* be specified.
*
TABLER	   MAR	   *,AR3       ; AR3 contains destination in data memory.
	   LAR	   AR3,#300h
	   RPT	   #127        ; Move 128 items to data memory block B1
	   TBLR    *+
	   RET
*
* This routine uses the TBLW instruction to move data memory to
* program memory.  The calling routine must contain the destination program
* memory address in the accumulator.
*
TABLEW	   MAR	   *,AR4       ; LARP = AR4.
	   LAR	   AR4,#380h   ; AR4 contains source address in data memory.
	   RPT	   #127        ; Move 128 items from data memory to program
	   TBLW    *+	       ; memory.
	   RET
*
* This routine uses the SMMR instruction to move data from a memory-mapped
* I/O port to local data memory. Note that 16 I/O ports are mapped in
* data page 0 of TMS320C5x memory map.
*
INPUT	   LDP	   #0
	   RPT	   #511        ; Input 512 values from port 51h to a table
	   SMMR    51h,800h    ; at 800h in data memory.
	   RET
*
* This routine uses the LMMR instruction to move data from local data space
* to a memory-mapped I/O port. Note that 16 I/O ports are mapped in data
* page 0 of TMS320C5x memory map.
*
OUTPUT	   LDP	   #0	       ; data page 0
	   RPT	   #63	       ; Output 64 values from a table at 800h in
	   LMMR    50h,800h    ; data memory to port 50h.
	   RET
