*******************************************************************************
*       (C) COPYRIGHT CURTIN UNIVERSITY, PERTH, WESTERN AUSTRALIA.            *   
*       Written by Ivar Siewert 930609A (February to June 1998).              *
*******************************************************************************
*                                                                             *
*  PROGRAM NAME: AUSTRALIAN CURRENCY NOTE READER FOR THE VISION IMPAIRED      *
*                                                                             *
*  DESCRIPTION: This program for the Texas Instruments TMS320C50 Digital      *
*               Signal Processor (DSP) is designed to recognise the           *
*               image of the clear window on Australian currency notes.       *
*               The image is produced by a Texas Instruments TSL218           *
*               linear array with a length of 512 pixels.                     *
*                                                                             *
*******************************************************************************

            .mmregs                 ;define global symbolic names for
                                    ; TMS320C5x registers.

*******************************************************************************
*                             GLOBAL CONSTANTS                                *
*******************************************************************************

DATA_PORT   .set  0050h             ;memory-mapped data Port Address 0.

*******************************************************************************
*                             GLOBAL VARIABLES                                *
*******************************************************************************

                  .ds         2b80h
FLAGS             .word       0           ;Explanation of flags:
                                          ;bit 0      SI flag
                  .ds         2b81h
SHIFT             .word       15
                  .ds         2b82h
BUFFER            .word       FFFFh
                  .ds         2b83h
LINE_BUFFER       .space      200h        ;storage for 512 pixels.
                  .ds         2b84h

*******************************************************************************
*                             INTERRUPT VECTORS                               *
*******************************************************************************

            ;set interrupt vectors

            .ps   0803h
            .word 098dh             ;store INT1 ISR address (CLKL signal).
            .ps   0802h
            .word 7980h             ;store branch instruction.
            .ps   0807h
            .word 0980h             ;store INT3 ISR address (SO signal).
            .ps   0806h
            .word 7980h             ;store branch instruction.
      
*******************************************************************************
*                         INTERRUPT SERVICE ROUTINES                          *
*******************************************************************************

            .ps   0980h             ;assemble code starting at this address.
            
            ;Serial Out SIGnal Interrupt Service Routine.
            ;This routine is triggered by external interrupt INT3.

SO_SIG_ISR  LDP   #0
            RPT   #50     
            OPL   #4000h,DATA_PORT  ;set DAT14, turn LEDs on.
            APL   #0BFFFh,DATA_PORT ;clear DAT14, turn LEDs off.
            LDP   #87
            OPL   #01h,FLAGS        ;set SI flag for next scan.
            LDP   #0
            SPLK  #01h,IFR
            RETE

;------------------------------------------------------------------------------

		;CLocK Low Interrupt Service Routine

            ;This routine is triggered by external interrupt INT1.
            ;The SI flag is first tested to see if this is the beginning of
            ; a new scan. If so, the SI signal is set high, the flag is

CLKL_ISR    LACC  DATA_PORT,3       ;get pixel and shift left to MSB of ACCL. 
            LDP   #87
            SACL  BUFFER            ;store pixel in buffer.
            BIT   FLAGS,15          ;test SI flag (bit 0).
            BCND  STORE_PIXEL,NTC   ;set SI signal if flag set, otherwise
                                    ; get pixel and store.
            APL   #0fffeh,FLAGS     ;clear SI flag.
            LAR   AR0,#0            ;reset pixel counter.
            LDP   #0
            OPL   #08000h,DATA_PORT ;set DAT15 high (SI signal).
HERE        BIT   IFR,15
            BCND  HERE,NTC
            RPT   #12
            NOP
            SPLK  #01h,IFR
            APL   #7FFFh,DATA_PORT  ;clear DAT15 (SI signal).
            RETE

STORE_PIXEL LAMM  AR0               ;load current pixel count.
            SAMM  TREG1             ;save bit storage position (bits 0-4).
            MAR   *-,AR1            ;decrement AR0 and change ARP to AR1.
            BSAR  4                 ;divide by 16, ACCL now holds the index
                                    ; to a particular word in the line buffer.
                                    ;the 4 LSBs of TREG1 gives the bit storage
                                    ; position of the pixel within that word.
            EXAR                    ;swap into ACCB.
            LACC  #LINE_BUFFER      ;load start address of line buffer.
            ORB                     ;both accumulators yield complete pointer.
            SAMM  AR1               ;store line buffer pointer in AR1.
            ;SMMR  AR7,#TREG1        ;recall and load bit storage position.
            LACT  BUFFER            ;reload pixel, with pixel shifted into
                                    ; correct position specified by TREG1.
            OR    *,AR1             ;OR pixel into correct word in line
                                    ; buffer, result in ACCL.
            SACL  *,0,AR1           ;store back into line buffer.
            MAR   *,AR0
            LDP   #0
		RETE

                      ;DON'T FORGET TO UPDATE INTERRUPT VECTORS.

******************************************************************************
*                             SUBROUTINES                                    *
******************************************************************************

            ;TMS320C50 Initialisation.

            ;CWSR = Wait-State Control Register
            ;PDWSR = Program Data Wait-State Register
            ;IOWSR = Input Output Wait-State Register
            ;PMST = Processor Mode STatus register
            ;PRD = timer PeRioD register
            ;TCR = Timer Control Register
            ;IFR = Interrupt Flag Register
            ;IMR = Interupt Mask Register

INITIALISE  LDP   #0                ;data pointer points to page 0.
            LAR   AR0,#0            ;reset pixel counter.
            SPLK  #0,CWSR           ;configure wait states.
            SPLK  #0,IOWSR          ;zero I/O wait states.
            SPLK  #0,PDWSR          ;zero program/data wait states.
            SPLK  #10h,TCR          ;timer stopped with no prescale.
            SPLK  #19,PRD           ;load timer for 1 us period (20MHz/[19+1]).
            OPL   #05h,IMR          ;unmask INT1 and INT3 external interrupts.
            SPLK  #0ffffh,IFR       ;clear any pending interrupts.
            CLRC  INTM              ;enable maskable interrupts.
            RET

;------------------------------------------------------------------------------

            ;explanation here

START_CCD   LDP   #87
            OPL   #01h,FLAGS        ;set SI flag (bit 0) to start scan.
            LDP   #0
            SPLK  #20h,TCR          ;reload and start timer.
                                    ;at least 12 cycles to purge pixels.
                                    ; then auto-calibrate.
            RET

;------------------------------------------------------------------------------

            ;explanation here

STOP_CCD    LDP   #0
            SPLK  #10h,TCR          ;stop timer immediately.
            RET

*******************************************************************************
*                             MAIN PROGRAM                                    *
*******************************************************************************

            ;explanation of main program

MAIN        CALL  INITIALISE
            CALL  START_CCD
            NOP
            NOP
BACK        BCND  BACK,UNC           ;go back and wait for another.
            NOP
            NOP

            .entry MAIN
            .end

;------------------------------------------------------------------------------


