
            .mmregs                 ;define global symbolic names for
                                    ; TMS320C5x registers.

*******************************************************************************
*                             GLOBAL CONSTANTS                                *
*******************************************************************************

DATA_PORT   .set  0050h             ;memory-mapped data Port Address 0.

*******************************************************************************
*                             GLOBAL VARIABLES                                *
*******************************************************************************

                  .ds         2b80h
LINE_BUFFER       .space      512         ;Temporary storage for 512 pixels.
                                          ;The buffer must start at the
                                          ; beginning of a data page.
                  .ds         2ba0h
FLAGS             .word       0           ;Explanation of flags:
                                          ;bit 0      SI flag
                  .ds         2ba1h
BUFFER            .word       00ffh


            .ps   0980h             ;assemble code starting at this address.

;------------------------------------------------------------------------------

CLKL_ISR    LACL  #01h   ;LAMM  DATA_PORT read in pixel data, pixel is bit 0.
            LDP   #87      
            SACL  BUFFER            ;store pixel in buffer.

STORE_PIXEL LAMM  AR0               ;load current pixel count.
            SAMM  TREG1             ;save bit storage position (bits 0-4).
            MAR   *-,AR1            ;decrement AR0 and change ARP to AR1.
            BSAR  4                 ;divide by 16, ACCL now holds the index
                                    ; to a particular word in the line buffer.
                                    ;the 4 LSBs of TREG1 gives the bit storage
                                    ; position of the pixel within that word.
            EXAR                    ;swap into ACCB.
            LACC  #LINE_BUFFER      ;load start address of line buffer.
            ORB                     ;both accumulators yield complete pointer.
            SAMM  AR1               ;store line buffer pointer in AR1.
            ;SMMR  AR7,#TREG1        ;recall and load bit storage position.
            LACT  BUFFER            ;reload pixel, with pixel shifted into
                                    ; correct position specified by TREG1.
            OR    *,AR1             ;OR pixel into correct word in line
                                    ; buffer, result in ACCL.
            SACL  *,0,AR1           ;store back into line buffer.
            MAR   *,AR0
            LDP   #0
            BANZ  CLKL_ISR,*,AR0
HERE        B     HERE

******************************************************************************
*                             SUBROUTINES                                    *
******************************************************************************

INITIALISE  LDP   #0                ;data pointer points to page 0.
            LAR   AR0,#511          ;reset pixel counter.
            LAR   AR1,#0            ;reset pointer.
            SPLK  #0,CWSR           ;configure wait states.
            SPLK  #0,IOWSR          ;zero I/O wait states.
            SPLK  #0,PDWSR          ;zero program/data wait states.
            SPLK  #10h,TCR          ;timer stopped with no prescale.
            SPLK  #19,PRD           ;load timer for 1 us period (20MHz/[19+1]).
            OPL   #05h,IMR          ;unmask INT1 and INT3 external interrupts.
            SPLK  #0ffffh,IFR       ;clear any pending interrupts.
            CLRC  INTM              ;enable maskable interrupts.
            RET

*******************************************************************************
*                             MAIN PROGRAM                                    *
*******************************************************************************

MAIN        CALL  INITIALISE
            B     CLKL_ISR

            .entry MAIN
            .end

;------------------------------------------------------------------------------


