/*
* Title:    Convert.c
*
* Purpose:  to translate a file from braille into print.
*
* Paul Blenkhorn
*
* Version:
*     1(a) - 27/8/92.  Extract main functions from old convert.c
*                      and take them outside.
*
*
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
 
#define         NUMBER_OF_CHARACTERS   256
#define         SPACE_BIT                8
#define         CAPITAL_INDICATOR        1
#define         NUMBER_INDICATOR       0xf
#define         LETTER_INDICATOR         3
#define         SPACE                    0
#define         NUMBITS               1024
#define         SHIFTBACK                3
#define         CAPITAL                  3

extern struct {        /* info for character set */
        unsigned char input_trans;      /* mapping for input chars */
        unsigned char to_up;            /* lower to upper case conversion */
        unsigned char data;             /* data for if chars are punc etc. */
        unsigned int hash;              /* position for hashing into tables */
} ch_info[NUMBER_OF_CHARACTERS];
 
#define         STR_SIZE               256
enum BOOLEAN    { FALSE, TRUE };

#define         NUL                    '\0'
 
FILE *infile, *outfile;

void add_to_output(unsigned char chr);
 
/* Function: convert_to_USSBC
   Convert the raw data into United States Standard Braille Code.
        Parameters:
                character that has just been read from file
        Return:
                Nil
*/
void convert_to_USSBC(unsigned char* chr)
{
static int numcaps = 0, isdigit = FALSE, isalpha = TRUE;
unsigned int exceptions[5] = {0x18,0x1c,0x54,0xcc,0xdc};
int    i;
printf("chr1 is %4x \n",*chr);
*chr /= NUMBITS;          /* This will move info into low byte */
printf("chr2 is %4x \n",*chr);
switch(*chr)
   {
   case  CAPITAL_INDICATOR:
         {
            numcaps ++;
            break;
         }
   case  NUMBER_INDICATOR:
         {
            isdigit = TRUE;
            isalpha = FALSE;
            break;
         }

   case  LETTER_INDICATOR:
         {
            isalpha = TRUE;
         }

   case  SPACE:
         {      /* Reset all values to default */
            numcaps = 0;
            isalpha = TRUE;
            isdigit = FALSE;
            break;
         }
   default:
         {
            if (isalpha)                               /* Know its a letter */
            {
               *chr *= 4;            /* Shift back to correct place */
               for (i = 0; i <= 5; i++)
               { /* These exceptions will not have the capital indicator in
                   front of them but need to have the extra added onto them */
                  if ( *chr == exceptions[i] )
                     *chr += 2;
               }
               if (numcaps)                          /* If a capital letter */
               {
                  *chr += CAPITAL;        /* Add on 3 to make USSBC capital */
                  if (isdigit)                     /* Must be Roman Numeral */
                     numcaps = 2;
                  if (numcaps == 1)            /* Only first letter capital */
                      numcaps = 0;     /* Reset,no more letters capitalised */
               } /* end if numcaps */
            } /* end if isalpha */
            else                                        /* Must be a number */
               *chr *= 2;   /* Can shift these left 1 to be in right places */
         } /* end default */

   } /* end switch */
printf("%4x ",*chr);
}


/* Function: read_line
   Read in a line from the input file.
        Parameters:
                buffer to read text into
        Return:
                whether end of file reached
*/
int read_line(unsigned char buffer[])
{
  register int i = 0;
  unsigned int chr;
  int new_line;
 
  do {
        fscanf(infile,"%d",chr);
        if (feof(infile)) {
                buffer[i] = NUL;
                return(EOF);
        }
printf("entering convert \n");
    convert_to_USSBC(&chr);
    new_line = (chr == '\n');
    chr = ch_info[chr].input_trans;  /* translate input ready for conversion */
    buffer[i++] = chr;
    if (i >= STR_SIZE)
             break;  /* don't let input line be too long */
  } while (!new_line);
  buffer[i] = NUL;
  return(TRUE);
}
 
main(int argc, char *argv[])
{
unsigned char
     input_txt[STR_SIZE];               /* put input text into this ready to translate it */
  unsigned char out_dat[256];          /* buffer for output data */
 
  /* copyright message */
  printf("Braille to Print Translator.  Version 1a.\n");
  printf("Copyright (c) UMIST, 1992.  All rights reserved.\n");
 
  /* get input and output file name from the command line */
  if((infile=fopen(argv[1],"r")) == NULL) {
        printf("Input file %s not found/n",argv[1]);
        exit(1);
  }
  if((outfile=fopen(argv[2],"w")) == NULL) {
        printf("Can't open output file %s/n",argv[2]);
        exit(2);
  }
 
  main_initialise("FLTAB");              /* load in tables and initialise */
 
  while (read_line(input_txt) != EOF) {  /* read lines up to EOF */
        if (strlen(input_txt))           /* if got text then convert */
                pas_convert(0,input_txt,out_dat);
        add_to_output('\n');             /* put out line feed */
        fprintf(outfile,"%s",out_dat);
  } /* while */
  return(0);
 
}
 
/************* Extra functions for interface between turbo pascal & c ******/
static int out_upto = 0;
static unsigned char *out_buff;           /* pointer to output buffer */
 
/* Function: pas_convert
   Translate the text in the buffer input_dat.
   Parameters:
        convert_to is set for how many chars, or character type to convert to
        input_dat is the input data
        output_dat is the output buffer
   Returns:
        number of characters converted
*/
int pas_convert(int convert_to, unsigned char input_dat[],unsigned char output_dat[])
{
      out_upto = 0;
      out_buff = output_dat;   /* use global to retain pointer for output buffer */
      return(convert(0,input_dat));
}
 
 
/* Function: add_to_output
           output a character to output file
   Parameters:
        chr is the character to add
   Returns:
        nothing
*/
void add_to_output(unsigned char chr)
{
#define NO_CAPS         0
#define CAPS_TEMP       1
#define CAPS_LOCK       2
 
  static char caps = NO_CAPS;
 
  if (chr == '^')
        caps++;
  else {
        if (( chr == ' ' ) || ( chr == '-' ))
                caps = NO_CAPS;
        if (caps) {
                chr = ch_info[chr].to_up;
                if (caps == CAPS_TEMP)
                        caps = NO_CAPS;
        }
        out_buff[out_upto++] = chr;
        out_buff[out_upto] = '\0';
        if (out_upto > 255)
                out_upto = 255;
  }
}
 